<?xml version="1.0" encoding="UTF-8"?>
<!-- =======================================================================
XRadar
Copyright (c) 2004, 2005, Kristoffer Kvam
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
Neither the name of Kristoffer Kvam nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

See licence.txt for dependancies to other open projects.
======================================================================= -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:redirect="http://xml.apache.org/xalan/redirect"
  	extension-element-prefixes="redirect">

	<xsl:output method="xml" encoding="UTF-8"/>

	<xsl:include href="xradar://stylesheets/util/api/filenameToUnix.xsl"/> 

	<xsl:param name="root-path"/>
    <xsl:param name="log"/>
    <xsl:param name="docs-home"/>
    <xsl:param name="radar-config-file"/>
    <xsl:param name="dynamics-configured"/>
    <xsl:param name="dynamics-root"/>

    <xsl:variable name="statics-root" select="'../../../'"/>

    <xsl:variable name="radar-config-doc" select="document($radar-config-file)"/>
    <xsl:variable name="metric-descriptions-doc"
                  select="'xradar://stylesheets/etc/data/metric-descriptions.xml'"/>
    <xsl:variable name="metric-descriptions"
                  select="document($metric-descriptions-doc)"/>

    <xsl:variable name="image-descriptions-doc"
                  select="'xradar://stylesheets/etc/data/image-descriptions.xml'"/>
    <xsl:variable name="image-descriptions"
                  select="document($image-descriptions-doc)"/>

<xsl:template match="/">

    <xsl:variable name="filename"><xsl:value-of select="concat($docs-home,'/xml/api/system.xml')"/></xsl:variable>
    <xsl:choose>
        <xsl:when test="$log = 'true'">

            <xsl:message>
        system: <xsl:value-of select="@system"/>
        log:    <xsl:value-of select="$log"/>
        docs-home:    <xsl:value-of select="$docs-home"/>
        radar-config:    <xsl:value-of select="$radar-config-file"/>
        dynamics-configured:    <xsl:value-of select="$dynamics-configured"/>
        dynamics-root:    <xsl:value-of select="$dynamics-root"/>
        filename:        <xsl:value-of select="$filename"/>
            </xsl:message>
        </xsl:when>
    </xsl:choose>



    <redirect:write file="{$filename}">
        <system>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="packages" mode="subsystem-report"/>
        </system>
    </redirect:write>

    <xsl:variable name="packages-element" select="packages"/>
    <xsl:for-each select="$radar-config-doc/radar-config/subsystems/subsystem">
        <xsl:variable name="subsystem-name" select="name"/>
            <redirect:write file="{$docs-home}/xml/api/subsystem/{$subsystem-name}.xml">
            <subsystem name="{$subsystem-name}">
                <xsl:apply-templates select="$packages-element" mode="subsystem-report">
                    <xsl:with-param name="subsystem-name" select="$subsystem-name"/>
                </xsl:apply-templates>
            </subsystem>
        </redirect:write>
    </xsl:for-each>

    <xsl:apply-templates select="packages/package" mode="package-report"/>
    <xsl:apply-templates select="packages/package/classes/class" mode="class-report"/>
</xsl:template>

<xsl:template match="packages" mode="subsystem-report">
    <xsl:param name="subsystem-name" select="''"/>
    <xsl:message>
        subsystem-name:<xsl:value-of select="$subsystem-name"/>
    </xsl:message>
    <xsl:variable name="level-indicator" select="'subsystem'"/>
    <xsl:variable name="level-name" select="$subsystem-name"/>
    <xsl:variable name="packages" select="count(package[starts-with(@subsystem,$subsystem-name)])"/>
    <images>
        <xsl:apply-templates select="$image-descriptions/image-descriptions/statics/svg[@subsystem-available='true']">
            <xsl:with-param name="root-name" select="$statics-root"/>
            <xsl:with-param name="level-indicator" select="$level-indicator"/>
            <xsl:with-param name="level-name" select="$level-name"/>
            <xsl:with-param name="type" select="'static'"/>
        </xsl:apply-templates>
        <xsl:if test="$dynamics-configured = 'true'">
            <xsl:apply-templates select="$image-descriptions/image-descriptions/dynamics/svg[@subsystem-available='true']">
                <xsl:with-param name="root-name" select="concat($dynamics-root,'/')"/>
                <xsl:with-param name="level-indicator" select="$level-indicator"/>
                <xsl:with-param name="level-name" select="$level-name"/>
                <xsl:with-param name="type" select="'dynamic'"/>
            </xsl:apply-templates>
        </xsl:if>
    </images>
    <statistics>
        <xsl:variable name="packages-element" select="current()"/>
        <defined-metrics>
	        <xsl:variable name="syst_quality_report_filename">
	        	<xsl:call-template name="filename2unix">
		        	<xsl:with-param name="url" select="concat($docs-home,'/xml/reports/system_quality_report.xml')"/>
		        </xsl:call-template>
	        </xsl:variable>
	        <!--  -->
			<xsl:choose>
				<xsl:when test="'true'">
			   		file: xml-plugin
			   		docs-home:<xsl:value-of select="$docs-home"/>
					system_quality_report: <xsl:value-of select="$syst_quality_report_filename"/>
				</xsl:when>	
			</xsl:choose>
		        <xsl:choose>
				<xsl:when test="$subsystem-name = ''">
					<xsl:variable name="system-quality-report" select="document(concat('file://',$syst_quality_report_filename))"/>
					<xsl:apply-templates select="$system-quality-report/metric"/>
		                </xsl:when>
		        <xsl:otherwise>       

							<xsl:variable name="system-quality-report">		        
					        	<xsl:choose>
					        		<xsl:when test="contains($docs-home,'\')">
			
											<xsl:call-template name="filename2unix">
												<xsl:with-param name="url" select="concat($docs-home,'/xml/reports/subsystem_quality_report_',normalize-space($subsystem-name),'.xml')"/>
											</xsl:call-template>
									</xsl:when>
									<xsl:otherwise>
										<xsl:value-of select="concat($docs-home,'/xml/reports/subsystem_quality_report_',normalize-space($subsystem-name),'.xml')"/>
									</xsl:otherwise>
								</xsl:choose>	
						</xsl:variable> 					
					<xsl:message>
						FIXME:<xsl:value-of select="$system-quality-report"/>
					</xsl:message>
					<!--
					FIXME: Never used ??? Disabled for now...
		    		<xsl:apply-templates select="$system-quality-report/metric"/> -->
				</xsl:otherwise> 
			</xsl:choose>
		</defined-metrics>
        <xsl:for-each select="$metric-descriptions/metric-descriptions/metric-definitions/group">
            <xsl:variable name="group-name" select="@name"/>
            <xsl:element name="{$group-name}">
                <xsl:for-each select="metric">
                    <xsl:variable name="metric-name" select="@name"/>
                    <xsl:variable name="divisor">
                        <xsl:choose>
                            <xsl:when test="@aggregate = 'mean'"><xsl:value-of select="$packages"/></xsl:when>
                            <xsl:otherwise>1</xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:element name="{$metric-name}">
                        <xsl:value-of
                            select="sum($packages-element/package[starts-with(@subsystem,$subsystem-name)]/statistics/node()[name() = $group-name]/node()[name() = $metric-name]) div $divisor"/>
                    </xsl:element>
                    </xsl:for-each>
            </xsl:element>
        </xsl:for-each>
    </statistics>
</xsl:template>


<xsl:template match="package" mode="package-report">
    <xsl:variable name="level-indicator" select="'package'"/>
    <xsl:variable name="level-name" select="@name"/>
    <redirect:write file="{$docs-home}/xml/api/package/{@name}.xml">
        <xsl:copy>
            <xsl:apply-templates select="@*"/>
            <images>
                <xsl:apply-templates select="$image-descriptions/image-descriptions/statics/svg[@package-available='true']">
                    <xsl:with-param name="root-name" select="$statics-root"/>
                    <xsl:with-param name="level-indicator" select="$level-indicator"/>
                    <xsl:with-param name="level-name" select="$level-name"/>
                    <xsl:with-param name="type" select="'static'"/>
                </xsl:apply-templates>
                <xsl:if test="$dynamics-configured = 'true'">
                    <xsl:apply-templates select="$image-descriptions/image-descriptions/dynamics/svg[@package-available='true']">
                        <xsl:with-param name="root-name" select="concat($dynamics-root,'/')"/>
                        <xsl:with-param name="level-indicator" select="$level-indicator"/>
                        <xsl:with-param name="level-name" select="$level-name"/>
                        <xsl:with-param name="type" select="'dynamic'"/>
                    </xsl:apply-templates>
                </xsl:if>
            </images>

            <svg id="1" name="Spider Graph" url="../../../img/spider-graph-package-{@name}.svg" descr=""/>
            <xsl:variable name="package-node" select="current()"/>
            <xsl:for-each select="statistics">
                <xsl:copy>
                    <xsl:apply-templates select="@*|node()"/>
                </xsl:copy>
            </xsl:for-each>
        </xsl:copy>
    </redirect:write>
</xsl:template>

<xsl:template match="svg">
    <xsl:param name="root-name" select="''"/>
    <xsl:param name="level-indicator" select="''"/>
    <xsl:param name="level-name" select="''"/>
    <xsl:param name="type"/>
    <xsl:copy>
        <xsl:attribute name="url">
            <xsl:value-of select="concat($root-name,'img/')"/>
            <xsl:for-each select="file-name/node()">
                <xsl:choose>
                    <xsl:when test="name() = 'text'"><xsl:value-of select="@value"/></xsl:when>
                    <xsl:when test="name() = 'level-indicator'"><xsl:value-of select="$level-indicator"/></xsl:when>
                    <xsl:when test="name() = 'level-name'"><xsl:value-of select="$level-name"/></xsl:when>
                </xsl:choose>
            </xsl:for-each>
            <xsl:value-of select="'.svg'"/>
        </xsl:attribute>
        <xsl:attribute name="type"><xsl:value-of select="$type"/></xsl:attribute>
        <xsl:apply-templates select="@name|@id"/>
        <xsl:apply-templates select="description"/>
    </xsl:copy>
</xsl:template>

<xsl:template match="class" mode="class-report">
    <redirect:write file="{$docs-home}/xml/api/class/{concat(../../@name,'.',@name)}.xml">
        <xsl:copy>
            <xsl:attribute name="package"><xsl:value-of select="../../@name"/></xsl:attribute>
            <xsl:attribute name="subsystem"><xsl:value-of select="../../@subsystem"/></xsl:attribute>
            <xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="node()"/>
        </xsl:copy>
    </redirect:write>
</xsl:template>

    <xsl:template match="functions">
        <methods><xsl:apply-templates select="@*|node()"/></methods>
    </xsl:template>

    <xsl:template match="@*|node()">
	   <xsl:copy>
	      <xsl:apply-templates select="@*|node()"/>
	   </xsl:copy>
	</xsl:template>

</xsl:stylesheet>
